

source("ks_endo.r")
source("blundell_powell.r")


rtexp <- function(n, rate=1, upper=3) {
 vals <- runif(n,min=0,max=pexp(upper,rate=1))
 qexp(vals,rate)
}


## Generate data. True (normalized) coefficient on endogenous
## regressor is equal to 1. 
n <- 100
z1 <- rnorm(n)
z2 <- (rtexp(n,1,upper=3)-.843)/.709
z3 <- rnorm(n)
v <- rnorm(n,0,sd=1)
y2 <- 1 +  2/3*z1 + 2/3*z3 + v + 1/3*z2 
u <- (rexp(n)-1) * sqrt(4.75)  + v + rnorm(n,0,sd=.25)
y1 <- y2 +  z2  - u  
y <- as.numeric(y1>0)
x <- matrix(c(y2,z2), ncol=2,byrow=FALSE)

# Calculate control function values via linear model (common to all estimators)
mod3 <- lm(y2 ~ z1  +  z3 + z2) 
v.hat <- resid(mod3)



# Probit MLE / Blundell-Smith / Rivers-Vuong
mod <- glm(y~x+v.hat,family=binomial(link="probit"))
coefs.probit <- mod$coef
coefs.probit[3]/coefs.probit[2]

# Linear Probability Model / 2SLS
mod2 <- lm(y~x+v.hat)
coefs.2sls <- mod2$coef
coefs.2sls[3]/coefs.2sls[2]

# Klein-Spady-Type Estimator with Endogeneity correction / Rothe (2008)
# Returns the following values: 
# 1) Coefficient estimate
# 2+3) Final bandwidth, if objective function was also maximized w.r.t. h.
# 4) log likelihood
# Uses probit estimates as starting values. 
# In practice, trying multiple starting values is recommended.
result <- ksendo(y, x, v.hat, init.beta=coefs.probit[3]/coefs.probit[2], init.h=rep(n^(-1/13),2), optim.bw=TRUE)
result

# Blundell-Powell-Estimator
bpe(y,x,v.hat,1.06*n^-.2)

